@@ -87,6 +87,7 @@ sub build_smtp_run {
 
     my @smtp_run_cmd = $self->toaster->supervised_do_not_edit_notice(1);
     push @smtp_run_cmd, $self->smtp_set_qmailqueue();
+    push @smtp_run_cmd, $self->smtp_get_simenv();
 
     $self->get_control_dir or return; # verify control directory exists
     $self->get_supervise_dir or return;
@@ -163,7 +164,7 @@ sub build_submit_run {
     return 1;
 }
 
-sub build_qmd_run {
+sub build_qmail_deliverable_run {
     my $self = shift;
     my $softlimit = $self->util->find_bin('softlimit');
     my $qmdd = $self->util->find_bin('qmail-deliverabled');
@@ -205,6 +206,13 @@ exec $tcpserver -vHRD 127.0.0.1 89 $vpopdir/bin/vpopmaild
     return 1;
 };
 
+sub build_qpsmtpd_run {
+    my $self = shift;
+# unless/until there's not settings in toaster-watcher.conf, we'll assume the
+# run file included with qpsmtpd is used
+    return 1;
+};
+
 sub check_rcpthosts {
     my ($self) = @_;
     my $qmaildir = $self->get_qmail_dir;
@@ -744,11 +752,11 @@ sub get_list_of_rbls {
 
         # test for custom sort key
         if ( $self->conf->{$key} > 1 ) {
-            $self->audit( "  sorted value $self->conf->{$key}" );
+            $self->audit( "  sorted value ".$self->conf->{$key} );
             @sorted[ $self->conf->{$key} - 2 ] = $1;
         }
         else {
-            $self->audit( "  unsorted, $self->conf->{$key}" );
+            $self->audit( "  unsorted, ".$self->conf->{$key} );
             push @unsorted, $1;
         }
     }
@@ -1041,8 +1049,9 @@ sub install_qmail_control_files {
         elsif ( $prot eq "send"   ) { $self->build_send_run   }
         elsif ( $prot eq "pop3"   ) { $self->build_pop3_run   }
         elsif ( $prot eq "submit" ) { $self->build_submit_run }
-        elsif ( $prot eq "qmail-deliverable" ) { $self->build_qmd_run }
+        elsif ( $prot eq "qmail-deliverable" ) { $self->build_qmail_deliverable_run }
         elsif ( $prot eq "vpopmaild" ) { $self->build_vpopmaild_run }
+        elsif ( $prot eq "qpsmtpd" ) { $self->build_qpsmtpd_run }
         else  { $self->error("I need help making run for $prot!"); };
     }
 }
@@ -1860,6 +1869,18 @@ sub send_stop {
     return 1;
 }
 
+sub smtp_get_simenv {
+    my $self = shift;
+
+    if ( $self->conf->{'simscan_debug'} ) {
+        $self->audit( "setting SIMSCAN_DEBUG");
+        return "SIMSCAN_DEBUG=1
+export SIMSCAN_DEBUG\n\n";
+    };
+
+    return '';
+};
+
 sub smtp_auth_enable {
     my $self = shift;
 
@@ -1890,7 +1911,7 @@ sub smtp_set_qmailqueue {
 
     if ( $self->conf->{'filtering_method'} ne "smtp" ) {
         $self->audit( "filtering_method != smtp, not setting QMAILQUEUE.");
-        return "";
+        return '';
     }
 
     # typically this will be simscan, qmail-scanner, or qmail-queue
@@ -1915,7 +1936,7 @@ You will continue to get this notice every 5 minutes until you fix this.\n";
 
     $self->audit( "  using $queue for QMAILQUEUE");
 
-    return "QMAILQUEUE=\"$queue\"\nexport QMAILQUEUE\n";
+    return "QMAILQUEUE=\"$queue\"\nexport QMAILQUEUE\n\n";
 }
 
 sub smtp_set_rbls {
